@@ -22,7 +22,7 @@
 #ifndef __ko_document_h__
 #define __ko_document_h__
 
-namespace std { };
+namespace std { }
 using namespace std;
 #include <qwmatrix.h>
 
@@ -55,12 +55,13 @@ class KoDocument : public KParts::ReadWritePart
 {
     Q_OBJECT
     Q_PROPERTY( QCString dcopObjectId READ dcopObjectId)
-    
+    Q_PROPERTY( bool backupFile READ backupFile WRITE setBackupFile )
+
 public:
 
     /**
      *  Constructor.
-     * The first 4  arguments are the same as the ones passed to KParts::Factory::createPart.
+     * The first 4 arguments are the same as the ones passed to KParts::Factory::createPart.
      *
      * @param parentWidget the parent widget, in case we create a wrapper widget
      *        (in single view mode).
@@ -70,8 +71,8 @@ public:
      * @param name is used to identify this document via DCOP so you may want to
      *        pass a meaningful name here which matches the pattern [A-Za-z_][A-Za-z_0-9]*.
      * @param singleViewMode determines whether the document may only have one view. In this case
-     *        the @param parent must be a QWidget derived class. KoDocument will then create a wrapper widget
-     *        (@ref KoViewWrapperWidget) which is a child of @param parentWidget.
+     *        the @p parent must be a QWidget derived class. KoDocument will then create a wrapper widget
+     *        (@ref KoViewWrapperWidget) which is a child of @p parentWidget.
      *        This widget can be retrieved by calling @ref #widget.
      */
     KoDocument( QWidget* parentWidget,
@@ -133,6 +134,30 @@ public:
      */
     virtual bool openURL( const KURL & url );
 
+    /**
+     * Opens the document given by @p url, without storing the URL
+     * in the KoDocument.
+     * Call this instead of openURL() to implement KoMainWindow's
+     * File --> Import feature.
+     *
+     * Note: This will call openURL(). To differentiate this from an ordinary
+     *       Open operation (in any reimplementation of openURL() or openFile())
+     *       call @ref isImporting().
+     */
+    bool import( const KURL &url );
+
+    /**
+     * Saves the document as @p url without changing the state of the
+     * KoDocument (URL, modified flag etc.). Call this instead of
+     * KParts::ReadWritePart::saveAs() to implement KoMainWindow's
+     * File --> Export feature.
+     *
+     * Note: This will call KoDocument::saveAs(). To differentiate this
+     *       from an ordinary Save operation (in any reimplementation of
+     *       saveFile()) call @ref isExporting().
+     */
+    bool exp0rt( const KURL &url );
+
     /**
      * Sets whether the document can be edited or is read only.
      * This recursively applied to all child documents and
@@ -144,7 +169,7 @@ public:
     /**
      * Used by KoApplication, when no document exists yet.
      *
-     * With the help of @param instance or @ref KApplication::instance() this
+     * With the help of @p instance or @ref KApplication::instance() this
      * method figures out which .desktop file matches this application. In this
      * file it searches for the "X-KDE-NativeMimeType" entry and returns it.
      *
@@ -167,6 +192,11 @@ public:
 
     enum { SaveAsKOffice1dot1 = 1, SaveAsDirectoryStore = 2 };
 
+    /**
+     * Returns the actual mimetype of the document
+     */
+    QCString mimeType() const;
+
     /**
      * Set the format in which the document should be saved.
      * This is called on loading, and in "save as", so you shouldn't
@@ -177,6 +207,22 @@ public:
     QCString outputMimeType() const;
     int specialOutputFlag() const;
 
+    /**
+     * Returns true if this document was the result of opening a foreign
+     * file format and if the user hasn't yet saved the document (in any
+     * format).
+     *
+     * Used by KoMainWindow to warn the user when s/he lazily presses
+     * CTRL+S to save in the same foreign format, putting all his/her
+     * formatting at risk (normally an export confirmation only comes up
+     * with Save As).
+     *
+     * @param exporting specifies whether this is the setting for a
+     * File --> Export or File --> Save/Save As operation.
+     */
+    bool confirmNonNativeSave( const bool exporting ) const;
+    void setConfirmNonNativeSave( const bool exporting, const bool on );
+
     /**
      * Sets the error message to be shown to the user (use i18n()!)
      * when loading or saving fails.
@@ -198,10 +244,19 @@ public:
      */
     virtual void addView( KoView *view );
 
+    /**
+     * Removes a view of the document.
+     */
     virtual void removeView( KoView *view );
 
+    /**
+     * @return a list of views this document is displayed in
+     */
     const QPtrList<KoView> & views() const;
 
+    /**
+     * @return number of views this document is displayed in
+     */
     int viewCount() const;
 
     /**
@@ -211,19 +266,19 @@ public:
 
     /**
      *  Find the most nested child document which contains the
-     *  questionable point. The point is in the coordinate system
+     *  requested point. The point is in the coordinate system
      *  of this part. If no child document contains this point, then
      *  a pointer to this document is returned.
      *
      *  This function has to be overloaded if the document features child documents.
      *
      *  @param matrix transforms points from the documents coordinate system
-     *         to the coordinate system of the questionable point.
+     *         to the coordinate system of the requested point.
      *  @param pos is in some unknown coordinate system, but the matrix can
      *         be used to transform a point of this parts coordinate system
      *         to the coordinate system of p.
      *
-     *  @return Pointer to the document, that was hit.
+     *  @return Pointer to the document which was hit.
      */
     virtual KoDocument *hitTest( const QPoint &pos, const QWMatrix &matrix = QWMatrix() );
 
@@ -240,6 +295,10 @@ public:
     virtual void paintEverything( QPainter &painter, const QRect &rect, bool transparent = false,
                                   KoView *view = 0L, double zoomX = 1.0, double zoomY = 1.0 );
 
+    /**
+     * Generates a preview picture of the document
+     * (to be used in the File Dialog).
+     */
     virtual QPixmap generatePreview( const QSize& size );
 
     /**
@@ -266,7 +325,8 @@ public:
      *
      *  @see #paintEverything #paintChildren #paintContent
      */
-    virtual void paintChild( KoDocumentChild *child, QPainter &painter, KoView *view, double zoomX = 1.0, double zoomY = 1.0 );
+    virtual void paintChild( KoDocumentChild *child, QPainter &painter, KoView *view,
+                             double zoomX = 1.0, double zoomY = 1.0 );
 
     /**
      *  Paints the data itself. Normally called by @ref paintEverthing. It does not
@@ -281,7 +341,8 @@ public:
      *
      *  @see #paintEverything
      */
-    virtual void paintContent( QPainter &painter, const QRect &rect, bool transparent = false, double zoomX = 1.0, double zoomY = 1.0 ) = 0;
+    virtual void paintContent( QPainter &painter, const QRect &rect, bool transparent = false,
+                               double zoomX = 1.0, double zoomY = 1.0 ) = 0;
 
     /**
      * Called by koApplication to check for an autosave file in $HOME
@@ -290,9 +351,23 @@ public:
 
     /**
      *  Initializes an empty document (display the template dialog!).
-     *  You have to overload this method to initalize all your document variables.
+     *  You have to overload this method to initialize all your document variables.
      */
-    virtual bool initDoc() = 0;
+    virtual bool initDoc() = 0; // BCI: pass a QWidget* parentWidget, to pass to KoTemplateChooseDia
+
+    // Who calls initDoc?
+    enum InitDocFlags { InitDocAppStarting, InitDocFileNew, InitDocFileClose, InitDocEmbedded };
+    /**
+     * Return some flags for initDoc (BCI: this should be merged with initDoc)
+     * Usually the app will want to
+     * - show the template dialog with 'everything' if InitDocAppStarting, InitDocFileClose or InitDocEmbedded
+     * - show the template dialog with 'templates only' if InitDocFileNew
+     */
+    InitDocFlags initDocFlags() const;
+    /**
+     * For KoMainWindow only - don't use!
+     */
+    void setInitDocFlags( InitDocFlags flags );
 
     /**
      *  Sets the modified flag on the document. This means that it has
@@ -302,7 +377,7 @@ public:
 
     /**
      *  Tells the document that its title has been modified, either because
-     *  the modified status changes (this is done by @ref setModified) or
+     *  the modified status changes (this is done by @ref #setModified) or
      *  because the URL or the document-info's title changed.
      */
     virtual void setTitleModified();
@@ -316,7 +391,7 @@ public:
      *  Sets the document to empty. Used after loading a template
      *  (which is not empty, but not the user's input).
      *
-     * @ref #isEmpty
+     *  @see #isEmpty
      */
     virtual void setEmpty() { m_bEmpty = true; }
 
@@ -329,7 +404,7 @@ public:
 
     /**
      *  Saves a document to a store.
-     *  You should not have to reimplement this - but call it in @ref saveChildren.
+     *  You should not have to reimplement this - but call it in @ref #saveChildren.
      */
     virtual bool saveToStore( KoStore* store, const QString& path );
 
@@ -369,7 +444,7 @@ public:
 
     /**
      *  Save the document. The default implementation is to call
-     *  @ref saveXML. This method exists only for applications that
+     *  @ref #saveXML. This method exists only for applications that
      *  don't use QDomDocument for saving, i.e. kword and kpresenter.
      */
     virtual bool saveToStream( QIODevice * dev );
@@ -428,7 +503,7 @@ public:
 
     /**
      * @return the KoDocumentChild associated with the given Document, but only if
-     *         "doc" is a direct child of this document.
+     *         @p doc is a direct child of this document.
      *
      * This is a convenience function. You could get the same result
      * by traversing the list returned by @ref #children.
@@ -459,27 +534,36 @@ public:
      */
     virtual void removeShell( KoMainWindow *shell );
 
+    /**
+     * @return the list of shells for the main window
+     */
     const QPtrList<KoMainWindow>& shells() const;
 
+    /**
+     * @return the number of shells for the main window
+     */
     int shellCount() const;
 
     /**
-     * Returns the list of all the currently opened documents
+     * @return the list of all the currently opened documents
      */
     static QPtrList<KoDocument> *documentList() { return s_documentList; }
 
     /**
      * Return a DCOP interface for this document
      * KOffice parts are strongly recommended to reimplement this method,
-     * so that their dcop interface provides more functionality than the basic KoDocumentIface
+     * so that their DCOP interface provides more functionality than the basic KoDocumentIface
      */
     virtual DCOPObject * dcopObject();
-    
+
     /**
-     * return the ID of the dcop interface for this document.
+     * @return the ID of the DCOP interface for this document.
      **/
     QCString dcopObjectId() const;
 
+    /**
+     * Signal the progress of operations such as loading or saving.
+     */
     void emitProgress( int value ) { emit sigProgress( value ); }
 
     bool isInOperation();
@@ -492,13 +576,112 @@ public:
      */
     virtual bool isStoredExtern();
 
+    /**
+     * @return the page layout associated with this document (margins, paper, etc).
+     *
+     * @see KoPageLayout
+     */
     KoPageLayout pageLayout() const { return m_pageLayout; }
-    
+
+    /**
+     * Performs a cleanup of unneeded backup files
+     */
     void removeAutoSaveFiles();
-    
+
+    void setBackupFile( bool _b );
+
+    bool backupFile()const;
+
+    /**
+     * Returns true if this document or any of its internal child documents are modified.
+     */
+    bool isModified();
+
+    int queryCloseExternalChildren();
+    int queryCloseDia();
+
+    /**
+     * Set when we do not want to save external children when saving our 'main' doc.
+     * This makes it possible to save 'main' doc + all its internal children first, then
+     * go on to save external children. Typically used by query close.
+     * Use:
+     *      doc->setDoNotSaveExtDoc();
+     *      doc->save();    // saves doc and its internal children,
+     *                            //also calls saveExternalChildren() which sets setDoNotSaveExtDoc(false)
+     *      doc->saveExternalChildren();
+     *
+     */
+    void setDoNotSaveExtDoc( bool on = true );
+
+    /**
+     * Sets the backup path of the document
+     */
+    void setBackupPath( const QString & _path );
+
+    /**
+     * @return path to the backup document
+     */
+    QString backupPath()const;
+
+    /**
+     * Indicates that this document is currently viewed
+     * and thus should control the title caption.
+     * Also resets current flag for all parents.
+     */
+    void setCurrent( bool on = true );
+
+    /**
+     * Sets current flag for this document and all its parents
+     */
+    void forceCurrent( bool on );
+    bool isCurrent() const;
+
+    void setTitleModified( const QString caption, bool mod );
+
+    /**
+     * Sets the document URL to empty URL
+     * KParts doesn't allow this, but KOffice apps have e.g. templates
+     * After using loadNativeFormat on a template, one wants
+     * to set the url to KURL()
+     */
+    void resetURL() { m_url = KURL(); m_file = QString::null; }
+
+    /**
+     * Set when you want an external embedded document to be stored internally
+     */
+    void setStoreInternal( bool i );
+
+    /**
+     * @return true when external embedded documents are stored internally
+     */
+    bool storeInternal() const;
+
+    bool hasExternURL();
+
+    /**
+     * Sets the document URL to @p url
+     * KParts doesn't really allow this, but it is needed for undo of setStoreInternal()
+     */
+    void setURL( const KURL& url ) { m_url = url; }
+
+    /**
+     * _Only_ use these functions to restore m_file (in KoMainWindow) after a
+     * failed save (remember to use setURL() to restore the URL as well).
+     *
+     * Do _not_ use these functions for any other purpose.
+     *
+     * @internal
+     */
+    QString &file() { return m_file; }
+
+    /**
+     * @internal
+     */
+    void setFile( const QString &file ) { m_file = file; }
+
 signals:
     /**
-     * This signal is emitted, if a direct or indirect child document changes
+     * This signal is emitted when a direct or indirect child document changes
      * and needs to be updated in all views.
      *
      * If one of your child documents emits the childChanged signal, then you may
@@ -507,7 +690,24 @@ signals:
      */
     void childChanged( KoDocumentChild *child );
 
-    void sigProgress(int value);
+    /**
+     * Progress info while loading or saving. The value is in percents (i.e. a number between 0 and 100)
+     * Your KoDocument-derived class should emit the signal now and then during load/save.
+     * KoMainWindow will take care of displaying a progress bar automatically.
+     */
+    void sigProgress( int value );
+
+    /**
+     * Emitted e.g. at the beginning of a save operation
+     * This is emitted by KoDocument and used by KoView to display a statusbar message
+     */
+    void sigStatusBarMessage( const QString& text );
+
+    /**
+     * Emitted e.g. at the end of a save operation
+     * This is emitted by KoDocument and used by KoView to clear the statusbar message
+     */
+    void sigClearStatusBarMessage();
 
     void sigBeginOperation();
     void sigEndOperation();
@@ -536,6 +736,22 @@ protected:
      */
     virtual bool saveFile();
 
+    /**
+     *  Returns whether or not the current openURL() or openFile() call is
+     *  actually an import operation (like File --> Import).
+     *  This is for informational purposes only.
+     */
+    bool isImporting() const;
+
+    /**
+     *  Returns whether or not the current saveFile() call is actually an export
+     *  operation (like File --> Export).
+     *  If this function returns true during saveFile() and you are changing
+     *  some sort of state, you _must_ restore it before the end of saveFile();
+     *  otherwise, File --> Export will not work properly.
+     */
+    bool isExporting() const;
+
     /**
      *  You need to overload this function if your document may contain
      *  embedded documents. This function is called to load embedded documents.
@@ -556,28 +772,9 @@ protected:
     virtual bool loadChildren( KoStore* );
 
     /**
-     *  Saves all children. If your document supports embedding, then you have
-     *  to overload this function. An implementation may look like this:
-     *  <PRE>
-     *  int i = 0;
-     *
-     *  QPtrListIterator<KoDocumentChild> it( children() );
-     *  for( ; it.current(); ++it ) {
-     *    KoDocument* childDoc = static_cast<KoDocumentChild*>(it.current())->document();
-     *    if ( childDoc )
-     *      if ( childDoc->isStoredExtern() )
-     *      {
-     *          if ( !childDoc->save() )
-     *              return FALSE;
-     *      }
-     *      else {
-     *          QString internURL = QString( "%1/%2" ).arg( _path ).arg( i++ );
-     *          if ( !childDoc->saveToStore( _store, internURL ) )
-     *              return FALSE;
-     *      }
-     *  }
-     *  return true;
-     *  </PRE>
+     *  Saves all internal children (only!).
+     *  @see saveExternalChildren if you have external children.
+     *  Returns true on success.
      */
     virtual bool saveChildren( KoStore* store );
 
@@ -600,14 +797,6 @@ protected:
      */
     virtual bool completeSaving( KoStore* store );
 
-    /**
-     * Sets the document URL to empty URL
-     * KParts doesn't allow this, but KOffice apps have e.g. templates
-     * After using loadNativeFormat on a template, one wants
-     * to set the url to KURL()
-     */
-    void resetURL() { m_url = KURL(); m_file = QString::null; }
-
     /**
      * Inserts the new child in the list of children and emits the
      * @ref #childChanged signal.
@@ -629,10 +818,18 @@ protected:
 
     KoPageLayout m_pageLayout;
 
+    /**
+     *  Saves all externally stored children.
+     *  Returns true on success.
+     * @see #saveChildren for internal children
+     */
+    bool saveExternalChildren(); // BCI: make this virtual
+
 private slots:
     void slotChildChanged( KoChild *c );
     void slotChildDestroyed();
     void slotAutoSave();
+    void slotStarted( KIO::Job* );
 
 private:
     void savePreview( KoStore* store );
